/* Keyboard parser mostly by Forrest Heller*/
module keyboard_parser(output reg char_recieved,
		     output wire parity_error, 
			 output reg [7:0] ascii,
			 input wire ps2_clock,
			 input wire ps2_data,
			 input wire clk);
		reg [7:0] last_char = 0;
		reg [7:0] char = 0;
		wire [7:0] ps2_byte;
		
		reg shift_is_pressed = 0;
		wire ps2_byte_recieved;
		ps2 kb (.char_recieved(ps2_byte_recieved),
			 .parity_error(parity_error),
			 .current_char(ps2_byte),
			 .ps2_clock(ps2_clock),
			 .ps2_data(ps2_data),
			 .clk(clk));
		always @(posedge clk)
			if (last_char == 8'hf0) begin
				if (char == 8'h12 || char == 8'h59)  
					shift_is_pressed <= 0;
				else
					shift_is_pressed <= shift_is_pressed;
			end else if (char == 8'h12 || char == 8'h59)
				shift_is_pressed <= 1;
			else
				shift_is_pressed <= shift_is_pressed;
		always @(posedge clk) begin
			if (ps2_byte_recieved == 1) begin
				char <= ps2_byte;
				last_char <= char;
			end else begin
				char <= char;
				last_char <= last_char;
			end
		end
		
		always @(posedge clk)
			if (ps2_byte_recieved == 1 && char != 8'hf0 && last_char != 8'hf0 && !(char == 8'h12 || char == 8'h59)) begin
		/* Too lazy to type myself.  Blantantly stolen from http://read.pudn.com/downloads76/sourcecode/embed/280310/verilog/ps2_keyboard.v__.htm*/		
			 casez ({shift_is_pressed,char}) 
				9'h?66 : ascii <= 8'h08;  // Backspace ("backspace" key) 
				9'h?0d : ascii <= 8'h09;  // Horizontal Tab 
				9'h?5a : ascii <= 8'h0d;  // Carriage return ("enter" key) 
				9'h?76 : ascii <= 8'h1b;  // Escape ("esc" key) 
				9'h?29 : ascii <= 8'h20;  // Space 
				9'h116 : ascii <= 8'h21;  // ! 
				9'h152 : ascii <= 8'h22;  // " 
				9'h126 : ascii <= 8'h23;  // # 
				9'h125 : ascii <= 8'h24;  // $ 
				9'h12e : ascii <= 8'h25;  // % 
				9'h13d : ascii <= 8'h26;  // & 
				9'h052 : ascii <= 8'h27;  // ' 
				9'h146 : ascii <= 8'h28;  // ( 
				9'h145 : ascii <= 8'h29;  // ) 
				9'h13e : ascii <= 8'h2a;  // * 
				9'h155 : ascii <= 8'h2b;  // + 
				9'h041 : ascii <= 8'h2c;  // , 
				9'h04e : ascii <= 8'h2d;  // - 
				9'h049 : ascii <= 8'h2e;  // . 
				9'h04a : ascii <= 8'h2f;  // / 
				9'h045 : ascii <= 8'h30;  // 0 
				9'h016 : ascii <= 8'h31;  // 1 
				9'h01e : ascii <= 8'h32;  // 2 
				9'h026 : ascii <= 8'h33;  // 3 
				9'h025 : ascii <= 8'h34;  // 4 
				9'h02e : ascii <= 8'h35;  // 5 
				9'h036 : ascii <= 8'h36;  // 6 
				9'h03d : ascii <= 8'h37;  // 7 
				9'h03e : ascii <= 8'h38;  // 8 
				9'h046 : ascii <= 8'h39;  // 9 
				9'h14c : ascii <= 8'h3a;  // : 
				9'h04c : ascii <= 8'h3b;  // ; 
				9'h141 : ascii <= 8'h3c;  // < 
				9'h055 : ascii <= 8'h3d;  // = 
				9'h149 : ascii <= 8'h3e;  // > 
				9'h14a : ascii <= 8'h3f;  // ? 
				9'h11e : ascii <= 8'h40;  // @ 
				9'h11c : ascii <= 8'h41;  // A 
				9'h132 : ascii <= 8'h42;  // B 
				9'h121 : ascii <= 8'h43;  // C 
				9'h123 : ascii <= 8'h44;  // D 
				9'h124 : ascii <= 8'h45;  // E 
				9'h12b : ascii <= 8'h46;  // F 
				9'h134 : ascii <= 8'h47;  // G 
				9'h133 : ascii <= 8'h48;  // H 
				9'h143 : ascii <= 8'h49;  // I 
				9'h13b : ascii <= 8'h4a;  // J 
				9'h142 : ascii <= 8'h4b;  // K 
				9'h14b : ascii <= 8'h4c;  // L 
				9'h13a : ascii <= 8'h4d;  // M 
				9'h131 : ascii <= 8'h4e;  // N 
				9'h144 : ascii <= 8'h4f;  // O 
				9'h14d : ascii <= 8'h50;  // P 
				9'h115 : ascii <= 8'h51;  // Q 
				9'h12d : ascii <= 8'h52;  // R 
				9'h11b : ascii <= 8'h53;  // S 
				9'h12c : ascii <= 8'h54;  // T 
				9'h13c : ascii <= 8'h55;  // U 
				9'h12a : ascii <= 8'h56;  // V 
				9'h11d : ascii <= 8'h57;  // W 
				9'h122 : ascii <= 8'h58;  // X 
				9'h135 : ascii <= 8'h59;  // Y 
				9'h11a : ascii <= 8'h5a;  // Z 
				9'h054 : ascii <= 8'h5b;  // [ 
				9'h05d : ascii <= 8'h5c;  // \ 
				9'h05b : ascii <= 8'h5d;  // ] 
				9'h136 : ascii <= 8'h5e;  // ^ 
				9'h14e : ascii <= 8'h5f;  // _     
				9'h00e : ascii <= 8'h60;  // ` 
				9'h01c : ascii <= 8'h61;  // a 
				9'h032 : ascii <= 8'h62;  // b 
				9'h021 : ascii <= 8'h63;  // c 
				9'h023 : ascii <= 8'h64;  // d 
				9'h024 : ascii <= 8'h65;  // e 
				9'h02b : ascii <= 8'h66;  // f 
				9'h034 : ascii <= 8'h67;  // g 
				9'h033 : ascii <= 8'h68;  // h 
				9'h043 : ascii <= 8'h69;  // i 
				9'h03b : ascii <= 8'h6a;  // j 
				9'h042 : ascii <= 8'h6b;  // k 
				9'h04b : ascii <= 8'h6c;  // l 
				9'h03a : ascii <= 8'h6d;  // m 
				9'h031 : ascii <= 8'h6e;  // n 
				9'h044 : ascii <= 8'h6f;  // o 
				9'h04d : ascii <= 8'h70;  // p 
				9'h015 : ascii <= 8'h71;  // q 
				9'h02d : ascii <= 8'h72;  // r 
				9'h01b : ascii <= 8'h73;  // s 
				9'h02c : ascii <= 8'h74;  // t 
				9'h03c : ascii <= 8'h75;  // u 
				9'h02a : ascii <= 8'h76;  // v 
				9'h01d : ascii <= 8'h77;  // w 
				9'h022 : ascii <= 8'h78;  // x 
				9'h035 : ascii <= 8'h79;  // y 
				9'h01a : ascii <= 8'h7a;  // z 
				9'h154 : ascii <= 8'h7b;  // { 
				9'h15d : ascii <= 8'h7c;  // | 
				9'h15b : ascii <= 8'h7d;  // } 
				9'h10e : ascii <= 8'h7e;  // ~ 
				9'h?71 : ascii <= 8'h7f;  // (Delete OR DEL on numeric keypad) 
				default : ascii <= (8'h78 - char);  // '.' used for unlisted characters. 
			  endcase
			  char_recieved <= 1;
			end else begin
				ascii <= ascii;
				char_recieved <= 0;
			end
endmodule
