/* LCD module Forrest Heller*/
	module lcd(output reg [7:0] lcd_data_bus,
output wire lcd_e,
output wire lcd_rw,
output wire lcd_rs,
output wire lcd_on,
output wire lcd_backlight,
input wire should_display_character,
input wire [7:0] character,
output wire busy,
input wire clock,
output wire [3:0] debug,
input wire should_clear_display,
input wire should_wrap_text);
reg [7:0] temporary_character;
reg has_temporary_character;
reg [3:0] state = 0;
reg [21:0] timer = 0;
reg [7:0] char_counter;
assign lcd_rs = ((state == 8) && has_temporary_character == 0);
assign lcd_rw = 0;
assign lcd_e = lcd_enabled;
assign busy = ((state != 4'h7) || has_temporary_character == 1 || should_clear_display == 1);
assign lcd_on = 1;
assign lcd_backlight = 1;
assign debug = state;	
reg lcd_enabled;
always @(posedge clock)
	case (state)
		0: begin
			lcd_data_bus <= 8'bzzzzzzzz;
			lcd_enabled <= 0;
		end
		1: begin
			lcd_data_bus <= 8'b00110000;
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		2: begin
			lcd_data_bus <= 8'b00110000;
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		3: begin
			lcd_data_bus <= 8'b00110000;
		    if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			
			end
		4: begin //function set
			lcd_data_bus <= 8'b00111000;
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		9: begin
			lcd_data_bus <= 8'b00001111;
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		5: begin //clear display
			lcd_data_bus <= 8'b00000001;
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		6: begin//entry mode
			lcd_data_bus <= 8'b00000110;
		    if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			end
		7:  begin //waiting/display character state
			if (should_clear_display == 1'b1) begin
				temporary_character <= 0;
				has_temporary_character <= 0;
			/* if we have a temporary character then use it*/
			end else if (has_temporary_character == 1'b1) begin
					lcd_data_bus <= temporary_character;
					temporary_character <= 0;
					has_temporary_character <= 0;
			/*we may not have a temporary character but we do have a character*/
			end else if (should_display_character == 1'b1) begin
				/* if we have displayed 16 characters then we should wrap the text*/
				if (should_wrap_text && char_counter == 16) begin
					/* fill the databus with a signal to set the display to posiion 0x40--the 2nd line*/
					lcd_data_bus <= 8'b11000000;
					has_temporary_character <= 1;
					temporary_character <= character;
				end else begin
					/* just put the character on the bus*/
					lcd_data_bus <= character;
				end
			end
			lcd_enabled <= 0;
			end
		8: begin //waiting mode
			if (timer <= 13) 
				lcd_enabled <= 1;
			else
				lcd_enabled <= 0;
			
			lcd_data_bus <= lcd_data_bus;
			end
		default: begin
			lcd_data_bus <= 8'bzzzzzzzz;			
			lcd_enabled <= 0;
		end

		endcase

always @(posedge clock)
	case (state)
		0: begin
			char_counter <= 0;
			// wait 15ms
			if (timer == 750000) begin
				state <= 1;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end
		end
		1: 	// wait for 4.1ms
			if (timer == 205000) begin
				state <= 2;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end

		2:
			// wait for 100us
			if (timer == 5000) begin
				state <= 3;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end
		3:
			// wait for 100us
			if (timer == 5000) begin
				state <= 4;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end

		4:
			// wait for 100us
			if (timer == 5000) begin
				state <= 9;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end
		9:
			// wait for 100us
			if (timer == 5000) begin
				state <= 5;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end

		5:
			// wait for 10ms
			if (timer == 500000) begin
				state <= 6;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end

		6:
			// wait for 100us
			if (timer == 5000) begin
				state <= 7;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end

		7: begin
			timer <= 0;
			// wait for character to display or to clear the display
			if (should_clear_display == 1) begin
				state <= 5;
				char_counter <= 0;
			end else if (should_display_character == 1 || has_temporary_character == 1'b1) begin
				//set the state to a waiting state
				state <= 8;
				char_counter <= char_counter + 8'h1;
			end else
				state <= state;
		end
		8:
			// wait for 100us
			if (timer == 5000) begin
				state <= 7;
				timer <= 0;
			end else begin
				timer <= timer+22'h1;
				state <= state;
			end
		
		default: begin
			timer <= timer;
			state <= state;
		end
		endcase
endmodule