/* Forrest Heller 2009*/
module main(
output wire [6:0] HEX0,
output wire [6:0] HEX1,
output wire [6:0] HEX2,
output wire [6:0] HEX3,
output wire [6:0] HEX4,
output wire [6:0] HEX5,
output wire [6:0] HEX6,
output wire [6:0] HEX7,
input wire PS2_CLK,
input wire PS2_DAT,
input wire CLOCK_50,
inout wire [7:0] LCD_DATA,
output wire LCD_ON,
output wire LCD_EN,
output wire LCD_RW,
output wire LCD_RS,
output wire LCD_BLON,
input wire [3:0] KEY,
input wire [17:0] SW,
output wire [7:0] LEDG,
output wire [8:0] LEDR
);
integer i;
wire [7:0] last_char_recieved;
reg [7:0] lcd_char;
reg lcd_display_char;
reg [7:0] characters [39:0];
wire [7:0] last_ps2_byte;
wire parity_error;
wire llparity_error;
wire char_recieved;
wire llbyte_recieved;
wire is_lcd_busy;
wire [3:0] debug;
reg [3:0] state;
wire [5:0] next_block_address = block_address + 6'h1;
reg [8:0] block_address;
reg lcd_should_clear_display;
lcd lcd_screen (.lcd_data_bus(LCD_DATA),
				.lcd_e(LCD_EN),				
				.lcd_rw(LCD_RW),
				.lcd_rs(LCD_RS),
				.lcd_on(LCD_ON),
				.lcd_backlight(LCD_BLON),
				.should_display_character(lcd_display_char),
				.character(lcd_char),
				.busy(is_lcd_busy),
				.clock(CLOCK_50),
				.debug(debug),
				.should_clear_display(lcd_should_clear_display | ~KEY[0]),
				.should_wrap_text(1'b1));
					

			 keyboard_parser kbp (.char_recieved(char_recieved),
			 .parity_error(parity_error),
			 .ascii(last_char_recieved),
			 .ps2_clock(PS2_CLK),
			 .ps2_data(PS2_DAT),
			 .clk(CLOCK_50));
			 
				
ps2 low_level_kb_recv(.char_recieved(llbyte_recieved),
			 .parity_error(llparity_error),
			 .current_char(last_ps2_byte),
			 .ps2_clock(PS2_CLK),
			 .ps2_data(PS2_DAT),
			 .clk(CLOCK_50));

wire [511:0] block;
reg [7:0] final_hash_string_mem [63:0];
wire [319:0] final_hash_string;
reg [7:0] block_mem [63:0];
wire [159:0] initial_hash;
wire [159:0] final_hash;
reg [5:0] character_pointer;
wire [3:0] hex [7:0];
divide m1(SW[3:0], SW[7:4], LEDG[7:4],LEDG[3:0]);

/* actual SHA1 module*/
assign initial_hash[31:0] = 32'h67452301;
assign initial_hash[63:32] = 32'hEFCDAB89;
assign initial_hash[95:64] = 32'h98BADCFE;
assign initial_hash[127:96] = 32'h10325476;
assign initial_hash[159:128] = 32'hC3D2E1F0;
sha1 hash(.block(block),
.current_hash(initial_hash),
.new_hash(final_hash));

/* this maps the final_hash to the final_hash_string */
hash_to_string hash_converter(.hash(final_hash),.hash_string(final_hash_string));

simple_hex s0(HEX0,hex[0]);
simple_hex s1(HEX1,hex[1]);
simple_hex s2(HEX2,hex[2]);
simple_hex s3(HEX3,hex[3]);
simple_hex s4(HEX4,hex[4]);
simple_hex s5(HEX5,hex[5]);
simple_hex s6(HEX6,hex[6]);
simple_hex s7(HEX7,hex[7]);

assign hex[0] = final_hash[131:128];
assign hex[1] = final_hash[135:132];
assign hex[2] = final_hash[139:136];
assign hex[3] = final_hash[143:140];
assign hex[4] = final_hash[147:144];
assign hex[5] = final_hash[151:148];
assign hex[6] = final_hash[155:152];
assign hex[7] = final_hash[159:156];
assign block[7:0] = block_mem[0];
assign block[15:8] = block_mem[1];
assign block[23:16] = block_mem[2];
assign block[31:24] = block_mem[3];
assign block[39:32] = block_mem[4];
assign block[47:40] = block_mem[5];
assign block[55:48] = block_mem[6];
assign block[63:56] = block_mem[7];
assign block[71:64] = block_mem[8];
assign block[79:72] = block_mem[9];
assign block[87:80] = block_mem[10];
assign block[95:88] = block_mem[11];
assign block[103:96] = block_mem[12];
assign block[111:104] = block_mem[13];
assign block[119:112] = block_mem[14];
assign block[127:120] = block_mem[15];
assign block[135:128] = block_mem[16];
assign block[143:136] = block_mem[17];
assign block[151:144] = block_mem[18];
assign block[159:152] = block_mem[19];
assign block[167:160] = block_mem[20];
assign block[175:168] = block_mem[21];
assign block[183:176] = block_mem[22];
assign block[191:184] = block_mem[23];
assign block[199:192] = block_mem[24];
assign block[207:200] = block_mem[25];
assign block[215:208] = block_mem[26];
assign block[223:216] = block_mem[27];
assign block[231:224] = block_mem[28];
assign block[239:232] = block_mem[29];
assign block[247:240] = block_mem[30];
assign block[255:248] = block_mem[31];
assign block[263:256] = block_mem[32];
assign block[271:264] = block_mem[33];
assign block[279:272] = block_mem[34];
assign block[287:280] = block_mem[35];
assign block[295:288] = block_mem[36];
assign block[303:296] = block_mem[37];
assign block[311:304] = block_mem[38];
assign block[319:312] = block_mem[39];
assign block[327:320] = block_mem[40];
assign block[335:328] = block_mem[41];
assign block[343:336] = block_mem[42];
assign block[351:344] = block_mem[43];
assign block[359:352] = block_mem[44];
assign block[367:360] = block_mem[45];
assign block[375:368] = block_mem[46];
assign block[383:376] = block_mem[47];
assign block[391:384] = block_mem[48];
assign block[399:392] = block_mem[49];
assign block[407:400] = block_mem[50];
assign block[415:408] = block_mem[51];
assign block[423:416] = block_mem[52];
assign block[431:424] = block_mem[53];
assign block[439:432] = block_mem[54];
assign block[447:440] = block_mem[55];
assign block[455:448] = block_mem[56];
assign block[463:456] = block_mem[57];
assign block[471:464] = block_mem[58];
assign block[479:472] = block_mem[59];
assign block[487:480] = block_mem[60];
assign block[495:488] = block_mem[61];
assign block[503:496] = block_mem[62];
assign block[511:504] = block_mem[63];


always @(posedge CLOCK_50)
	case (state) 
	0: begin
		lcd_should_clear_display <= 1;
		state <= 1;
	end
	1: begin
		character_pointer <= 0;
		lcd_display_char <= 0;
		lcd_should_clear_display <= 0;
		if (SW[16] == 1'b1 && SW[1] == 1'b1) 
			state <= 2;
		else
			for (i = 0; i < 64; i = i + 1) 
				final_hash_string_mem[i] <= 8'h0;
	end
	2: begin
		final_hash_string_mem[0] <= final_hash_string[7:0];
		final_hash_string_mem[1] <= final_hash_string[15:8];
		final_hash_string_mem[2] <= final_hash_string[23:16];
		final_hash_string_mem[3] <= final_hash_string[31:24];
		final_hash_string_mem[4] <= final_hash_string[39:32];
		final_hash_string_mem[5] <= final_hash_string[47:40];
		final_hash_string_mem[6] <= final_hash_string[55:48];
		final_hash_string_mem[7] <= final_hash_string[63:56];
		final_hash_string_mem[8] <= final_hash_string[71:64];
		final_hash_string_mem[9] <= final_hash_string[79:72];
		final_hash_string_mem[10] <= final_hash_string[87:80];
		final_hash_string_mem[11] <= final_hash_string[95:88];
		final_hash_string_mem[12] <= final_hash_string[103:96];
		final_hash_string_mem[13] <= final_hash_string[111:104];
		final_hash_string_mem[14] <= final_hash_string[119:112];
		final_hash_string_mem[15] <= final_hash_string[127:120];
		final_hash_string_mem[16] <= final_hash_string[135:128];
		final_hash_string_mem[17] <= final_hash_string[143:136];
		final_hash_string_mem[18] <= final_hash_string[151:144];
		final_hash_string_mem[19] <= final_hash_string[159:152];
		final_hash_string_mem[20] <= final_hash_string[167:160];
		final_hash_string_mem[21] <= final_hash_string[175:168];
		final_hash_string_mem[22] <= final_hash_string[183:176];
		final_hash_string_mem[23] <= final_hash_string[191:184];
		final_hash_string_mem[24] <= final_hash_string[199:192];
		final_hash_string_mem[25] <= final_hash_string[207:200];
		final_hash_string_mem[26] <= final_hash_string[215:208];
		final_hash_string_mem[27] <= final_hash_string[223:216];
		final_hash_string_mem[28] <= final_hash_string[231:224];
		final_hash_string_mem[29] <= final_hash_string[239:232];
		final_hash_string_mem[30] <= final_hash_string[247:240];
		final_hash_string_mem[31] <= final_hash_string[255:248];
		final_hash_string_mem[32] <= final_hash_string[263:256];
		final_hash_string_mem[33] <= final_hash_string[271:264];
		final_hash_string_mem[34] <= final_hash_string[279:272];
		final_hash_string_mem[35] <= final_hash_string[287:280];
		final_hash_string_mem[36] <= final_hash_string[295:288];
		final_hash_string_mem[37] <= final_hash_string[303:296];
		final_hash_string_mem[38] <= final_hash_string[311:304];
		final_hash_string_mem[39] <= final_hash_string[319:312];
	end
	3: begin
		if (is_lcd_busy == 0) begin
			if (character_pointer == 6'd32) begin
				state <= 4;
				character_pointer <= 0;
				lcd_display_char <= 0;
				lcd_char <= 0;
			end else begin
				character_pointer <= character_pointer + 6'h1;	
				lcd_char <= final_hash_string_mem[character_pointer];
				lcd_display_char <= 1;
			end
		end
	end
	4: 
		if (SW[16] == 1'b0) 
			state <= 0;
	default:
		state <= 0;
	endcase
always @(posedge CLOCK_50)
	case (state) 
		0: begin
			block_address <= 0;
			for(i = 0; i < 64; i = i + 1)
				block_mem[i] <= 0;
		end
		1: begin
			if (SW[1] == 1'b1) begin
				block_mem[block_address] <= 8'h80;
				block_mem[63] <= block_address << 8'h3;
			end else if (char_recieved == 1'b1) begin
				block_mem[block_address] <= last_char_recieved;
				block_address <= block_address + 6'h1;
			end
		end
		endcase
endmodule
